<?php
	include 'compiler/Blueprint.php';
	include 'editor/ui.php';
	
	function renderComponents()
	{
		echo "Компоненты:";
		echo "<div class='listbox'>";
		
		UI::handleForm(array("group"), function($args)
		{
			$_SESSION["group"] = $args["group"];
		});
		
		if(isset($_SESSION["group"]) && strlen($_SESSION["group"]))
		{
			global $f;
			
			echo "<a class='listbox-item' href='?dir=$f&group='><img src='editor/img/dir.png' /> К группам</a>";
		
			$src = NodeList::enumFactories($_SESSION["group"]);
			$even = false;
			
			foreach($src as $factory)
			{
				$fname = $factory->getName();
				$ficon = $factory->getIcon();
				
				if(!file_exists("editor/img/$ficon"))
					$ficon = "default.png";
				
				echo "<a class='listbox-item' href='?dir=$f&new=$fname'><img src='editor/img/$ficon' /> &raquo; $fname</a>";
			}
		}
		else
		{
			global $f;
			
			$src = NodeList::enumFactoryGroups();
			
			foreach($src as $group)
			{
				echo "<a class='listbox-item' href='?dir=$f&group=$group'>&raquo; $group</a>";
			}
		}
		
		echo "</div>";
	}
	
	function renderNodeAttributes($node, $factory)
	{
		$attrs = $factory->queryAttributeList();
		
		foreach($attrs as $attr)
		{
			$required = isset($attr["required"]) ? "<span style='color: red'>*</span>" : "";
			$attrName = $attr["name"];
			$attrName[0] = strtoupper($attrName[0]);
			
			echo "$attrName$required:<br>";
			
			if(isset($attr["enum"]))
			{
				echo "<select name='$attr[name]'>";
				
				foreach($attr["enum"] as $val)
				{			
					if(isset($node) && $node->getAttribute("$attr[name]") == $val)
						echo "<option selected value='$val'>$val</option>";
					else
						echo "<option value='$val'>$val</option>";
				}
				
				echo "</select><br><br>";
			}
			else
			{
				if(isset($node))
					$attrVal = $node->getAttribute($attr["name"]);
				else
					$attrVal = "";
				
				if(isset($attr["big"]))
					echo "<textarea name='$attr[name]'>$attrVal</textarea><br><br>";
				else
					echo "<input name='$attr[name]' value='$attrVal' /><br><br>";
			}
		}
	}
	
	function colorFromTabCount($tabCount)
	{
		switch($tabCount)
		{
			case 0: return "#20b286";
			case 1: return "#d64541";
			case 2: return "#f4f776";
			case 3: return "#736598";
			case 4: return "#f9bf3b";
			default: return "#3a539b";
		}
	}
	
	function renderNodes()
	{
		global $f, $blueprint;
		
		$tabCount = 0;
		
		echo "Программа:";
		echo "<div class='listbox'>";
		
		$nodes = $blueprint->getNodes();
		$even = false;
		
		for($i = 0; $i < sizeof($nodes); $i++)
		{
			if(!$nodes[$i])
				continue;
			
			$desc = $nodes[$i]->getDescription();
			$icon = $nodes[$i]->getFactory()->getIcon();
			
			echo "<div class='listbox-item "  . ($even ? "even" : "") .  "' href='?dir=$f&edit=$i'>";
			
			if($nodes[$i]->getFactory()->getName() == "BlockEnd")
				$tabCount--;
			
			for($j = 0; $j < $tabCount; $j++)
				echo "&nbsp;";
			
			if($tabCount > 0)
				echo "&raquo;";
			
			$color = colorFromTabCount($tabCount);
			
			if($nodes[$i]->getFactory()->getName() == "BlockStart")
				$tabCount++;
			
			echo "	<span style='color: $color'>$i.</span> <img src='editor/img/$icon' /> $desc <br><br>
					<a style='margin-left: 55px' href='?dir=$f&moveup=$i'><img src='editor/img/up-arrow.png' /></a>
					<a style='margin-left: 5px' href='?dir=$f&movedown=$i'><img src='editor/img/down-arrow.png' /></a>
					<a style='margin-left: 5px' href='?dir=$f&edit=$i'><img src='editor/img/edit.png' /></a>
					<a style='margin-left: 5px' href='?dir=$f&remove=$i'><img src='editor/img/delete.png' /></a>
					<a style='margin-left: 5px' href='?dir=$f&point=$i'><img src='editor/img/finger.png' /></a>
				 </div>";
			
			if($i == $_SESSION["pointer"])
				echo "<hr class='pointer' />";
				 
			$even = !$even;
		}
		
		echo "</div>";
	}
	
	echo "<div class='content'>";
	
	if($f = UI::selectFile())
	{
		$blueprint = new Blueprint();
		$result = $blueprint->load($f);
		
		if($result != Blueprint::E_OK)
		{
			echo "Ошибка при загрузке программы \"$f\" (код ошибки: $result)<br><br>";
			
			echo "<a href='?editor.php'>К выбору файлов</a>";
		}
		else
		{
			if(!isset($_SESSION["pointer"]))
				$_SESSION["pointer"] = 0;
			
			if(isset($_SESSION["pointer"]) && !$blueprint->getNode($_SESSION["pointer"]))
				$_SESSION["pointer"] = 0;
			
			// Create node
			UI::handleForm(array("new"), function($args, $arg)
			{
				global $f;
				
				echo "Создание компонента $args[new]<br><br>";
				
				$factory = NodeList::findFactory($args["new"]);
				
				if(!$factory)
					return;
				
				$attrs = $factory->queryAttributeList();
					
				$attrsArray = array("__name");
				
				// Collect attribute names for handling form
				foreach($attrs as $attr)
					$attrsArray[] = $attr["name"];
				
				UI::handleForm($attrsArray, function($args, $arg)
				{
					global $blueprint;
					
					$node = $blueprint->createNode($args["__name"]); // Node is always created at end of the list
					
					$_SESSION["pointer"]++;
					
					while($blueprint->indexOf($node) > $_SESSION["pointer"])
						$blueprint->moveNode($node, Blueprint::MOVE_UP);
					
					foreach($args as $key => $val)
					{
						$node->setAttribute($key, $val);
					}
				}, null, null, "POST");
				
				echo "<form action='?dir=$f&new=$args[new]' method='POST'>";
				
				renderNodeAttributes(null, $factory);
				
				echo "<input type='hidden' name='__name' value='$args[new]' />";
				echo "<input type='submit' value='Создать' />";
				echo "</form>";
			});
			
			// Set pointer
			UI::handleForm(array("point"), function($args)
			{
				global $blueprint;
				
				if(isset($args["point"]) && $blueprint->getNode($args["point"]))
					$_SESSION["pointer"] = $args["point"];
			});
			
			// Edit node
			UI::handleForm(array("edit"), function($args, $arg)
			{
				global $f, $blueprint;
				
				$node = $blueprint->getNode($args["edit"]);
				
				if($node)
				{
					$attrs = $node->getFactory()->queryAttributeList();
					
					$attrsArray = array("__editid");
					
					// Collect attribute names for handling form
					foreach($attrs as $attr)
						$attrsArray[] = $attr["name"];
					
					UI::handleForm($attrsArray, function($args, $arg)
					{	
						$editid = $args["__editid"];
						
						foreach($args as $key => $val)
						{
							if($key == "__editid")
								continue;
							
							$arg->setAttribute($key, $val);
						}
					},
					null, $node, "POST");
					
					echo "Редактирование узла #$args[edit](помеченные <span style='color: red'>*</span> обязательны):";
					echo "<form action='?dir=$f&edit=$args[edit]&save' method='POST'><br>";
					
					renderNodeAttributes($node, $node->getFactory());
					
					echo "<input type='hidden' name='__editid' value='$args[edit]' />";
					echo "<input type='submit' value='Сохранить' /><br><br>";
					echo "<a href='?dir=$f'>Назад</a>";
					
					echo "</form>";
				}
				else
				{
					echo "Неверный ID узла";
				}
			});
			
			// Remove node
			UI::handleForm(array("remove"), function($args)
			{
				global $blueprint;
				
				$node = $blueprint->getNode($args["remove"]);
				
				if($node)
					$blueprint->removeNode($node);
			}, null, null);
			
			// Move node
			UI::handleForm(array("moveup"), function($args)
			{
				global $blueprint;
				
				$node = $blueprint->getNode($args["moveup"]);
				
				if($node)
					$blueprint->moveNode($node, Blueprint::MOVE_UP);
			});
			
			UI::handleForm(array("movedown"), function($args)
			{
				global $blueprint;
				
				$node = $blueprint->getNode($args["movedown"]);
				
				if($node)
					$blueprint->moveNode($node, Blueprint::MOVE_DOWN);
			});
			
			renderComponents();
			
			echo "<br><br>";
			
			renderNodes();
			
			echo "<br><br>";
			
			$path = substr($f, 0, strripos($f, "."));
			
			$blueprint->setName(str_replace(" ", "_", pathinfo($path, PATHINFO_BASENAME)));
			$blueprint->setMain(true);
			
			// Compile
			UI::handleForm(array("compile"), function($args, $path)
			{
				global $blueprint;
				
				if(file_exists("$path.php"))
					unlink("$path.php");
				
				$blueprint->compile();
				
				file_put_contents("$path.php", $blueprint->getOutput());
				
				$_SESSION["buildStatus"] = $blueprint->getErrorMessage();
			}, null, $path);
			
			echo "<a class='button' href='?dir=$f&compile'>Скомпилировать</a> ";
			
			if(file_exists("$path.php"))
				echo "<a class='button' href='$path.php'>Запустить</a><br><br>";
			
			if(isset($_SESSION["buildStatus"]))
			{
				echo "<br><br>Лог последней компиляции:<br>";
				echo "<code>";
				echo "$_SESSION[buildStatus]";
				echo "</code>";
			}
			
			$blueprint->save($f);
			
			echo "<br><a href='editor.php'>Выбрать файл</a>";
		}
	}
	
	echo "<br><a href='menu.php'>В меню</a><br>";
	
	echo "</div>";
	
	include 'editor/foot.php';