<?php
	
	class CreateArrayNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Initialize array <b>%name</b>(%initializer)");
		}
	
		public function process()
		{
			return $this->format("$%name = array(%initializer);");
		}
		
	}
	
	class GetArrayLengthNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Store array <b>%array</b> size in variable <b>%variable</b>");
		}
	
		public function process()
		{
			return $this->format("$%variable = sizeof($%array);");
		}
		
	}
	
	class CopyArrayNode extends BaseNode
	{	
	
		public function process()
		{
			return $this->format("
				if(is_array($%from) && is_array($%to))
				{
					foreach($%from as $node_iterator_key => $node_iterator_value)
						$%to[$node_iterator_key] = $%from[$node_iterator_key];
				}
			");
		}
		
	}
	
	class GetArrayValueNode extends BaseNode
	{

		public function getDescription()
		{
			return $this->format("Store array <b>%array</b> key <b>%index</b> value in <b>%variable</b>");
		}
	
		public function process()
		{
			return $this->format("$%variable = $%array[%index];");
		}
		
	}
	
	class SetArrayValueNode extends BaseNode
	{

		public function getDescription()
		{
			return $this->format("Set array <b>%array</b> key <b>%index</b> value <b>%value</b>");
		}
	
		public function process()
		{
			return $this->format("$%array[%index] = %value;");
		}
		
	}
	
	class ClearArrayNode extends BaseNode
	{
	
		public function getDescription()
		{
			return $this->format("Clear array <b>%array</b>");
		}
	
		public function process()
		{
			return $this->format("%array = array();");
		}
		
	}
	
	
	class CreateArrayNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Create array";
		}
		
		public function getGroup()
		{
			return "Arrays";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new CreateArrayNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "name", "required" => true),
				array("name" => "initializer", "big" => true)
			);
		}
	}
	
	class GetArrayLengthNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Store array length";
		}
		
		public function getGroup()
		{
			return "Arrays";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new GetArrayLengthNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "variable", "required" => true),
				array("name" => "array", "required" => true)
			);
		}
	}
	
	class CopyArrayNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Copy array";
		}
		
		public function getGroup()
		{
			return "Arrays";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new CopyArrayNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "from", "required" => true),
				array("name" => "to", "required" => true)
			);
		}
	}
	
	class GetArrayValueNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Store array value";
		}
		
		public function getGroup()
		{
			return "Arrays";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new GetArrayValueNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "array", "required" => true),
				array("name" => "index", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class SetArrayValueNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Set array value";
		}
		
		public function getGroup()
		{
			return "Arrays";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new SetArrayValueNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "array", "required" => true),
				array("name" => "index", "required" => true),
				array("name" => "value", "required" => true)
			);
		}
	}
	
	class ClearArrayNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Clear array";
		}
		
		public function getGroup()
		{
			return "Arrays";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new ClearArrayNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "array", "required" => true)
			);
		}
	}
	
	NodeList::registerFactory(array(new CreateArrayNodeFactory, new GetArrayLengthNodeFactory, new ClearArrayNodeFactory, new SetArrayValueNodeFactory, new GetArrayValueNodeFactory, new CopyArrayNodeFactory));