<?php
	
	class ReadWholeFileNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Read whole file <b>%name</b> in variable <b>%variable</b>");
		}
	
		public function process()
		{
			return $this->format("$%variable = file_get_contents(%name);");
		}
		
	}
	
	class WriteContentsFileNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Write content <b>%content</b> to file <b>%name</b>");
		}
	
		public function process()
		{
			return $this->format("file_put_contents(%name, %content);");
		}
		
	}
	
	class OpenFileNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Open file <b>%handle</b>(mode: <b>%mode</b>) and store handle in <b>%variable</b>");
		}
	
		public function process()
		{
			return $this->format("$%variable = fopen(%handle, %mode);");
		}
		
	}
	
	class ReadFileNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Read string from file <b>%handle</b> and store in <b>%variable</b>");
		}
	
		public function process()
		{
			return $this->format("
				if($%handle && !feof($%handle))
					$%variable = fgets($%handle);
			");
		}
		
	}
	
	class WriteFileNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Write string <b>%content</b> to file <b>%handle</b>");
		}
	
		public function process()
		{
			return $this->format("if($%handle)fputs($%handle, %content . \"\\r\\n\");");
		}
		
	}
	
	class StoreFileFeofNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Check, if file is readed and store result in <b>%variable</b>");
		}
	
		public function process()
		{
			return $this->format("
				if($%handle)
					$%variable = feof($%handle);
			");
		}
		
	}
	
	class CloseFileNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Close file <b>%handle</b>");
		}
	
		public function process()
		{
			return $this->format("
				if($%handle)
					fclose($%handle);
			");
		}
		
	}
	
	class CheckFileExistenceNode extends BaseNode
	{
		
		public function getDescription()
		{
			return $this->format("Check if file <b>%name</b> exists and store result in <b>%variable</b>");
		}
	
		public function process()
		{
			return $this->format("$%variable = file_exists(%name)");
		}
		
	}
	
	
	class ReadWholeFileNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Read whole file";
		}
		
		public function getGroup()
		{
			return "Files";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new ReadWholeFileNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "name", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class StoreFileFeofNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Check end of file";
		}
		
		public function getGroup()
		{
			return "Files";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StoreFileFeofNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "handle", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class WriteContentsFileNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Write contents to file";
		}
		
		public function getGroup()
		{
			return "Files";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new WriteContentsFileNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "name", "required" => true),
				array("name" => "content", "required" => true)
			);
		}
	}
	
	class OpenFileNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Open file";
		}
		
		public function getGroup()
		{
			return "Files";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new OpenFileNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "handle", "required" => true),
				array("name" => "mode", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class ReadFileNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Read file";
		}
		
		public function getGroup()
		{
			return "Files";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new ReadFileNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "handle", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class WriteFileNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Write file";
		}
		
		public function getGroup()
		{
			return "Files";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new WriteFileNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "handle", "required" => true),
				array("name" => "content", "required" => true)
			);
		}
	}
	
	class CloseFileNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Close file";
		}
		
		public function getGroup()
		{
			return "Files";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new CloseFileNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "handle", "required" => true)
			);
		}
	}
	
	class CheckFileExistenceNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Check file exists";
		}
		
		public function getGroup()
		{
			return "Files";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new CheckFileExistenceNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "name", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	NodeList::registerFactory(array(new WriteContentsFileNodeFactory, new ReadWholeFileNodeFactory, new CheckFileExistenceNodeFactory));
	NodeList::registerFactory(array(new OpenFileNodeFactory, new ReadFileNodeFactory, new WriteFileNodeFactory, new StoreFileFeofNodeFactory, new CloseFileNodeFactory));