<?php
	
	class StoreTimeNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Store current time in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = time();");
		}
	}
	
	class StoreTimeNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Get time";
		}
		
		public function getGroup()
		{
			return "System";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StoreTimeNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "variable", "required" => true),
			);
		}
	}
	
	class StoreTimeStringNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Store current time as string in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = date('d.m.y H:i');");
		}
	}
	
	class StoreTimeStringNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Get time as string";
		}
		
		public function getGroup()
		{
			return "System";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StoreTimeStringNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "variable", "required" => true),
			);
		}
	}
	
	class ChangeCurrentDirectoryNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Change current working directory to <b>%path</b>");
		}
		
		public function process()
		{
			return $this->format("chdir(%path);");
		}
	}
	
	class ChangeCurrentDirectoryNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Change working directory";
		}
		
		public function getGroup()
		{
			return "System";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new ChangeCurrentDirectoryNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "directory", "required" => true)
			);
		}
	}
	
	class StoreCurrentDirectoryNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Store current working directory in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = getcwd();");
		}
	}
	
	class StoreCurrentDirectoryNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Get current working directory";
		}
		
		public function getGroup()
		{
			return "System";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StoreCurrentDirectoryNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "variable", "required" => true),
			);
		}
	}
	
	class StoreSystemVariableNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Store system variable <b>%system_variable</b> in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = getenv(%system_variable);");
		}
	}
	
	class StoreSystemVariableNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Get system variable";
		}
		
		public function getGroup()
		{
			return "System";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StoreSystemVariableNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "system variable", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	NodeList::registerFactory(array(new StoreTimeNodeFactory, new StoreTimeStringNodeFactory));
	NodeList::registerFactory(array(new ChangeCurrentDirectoryNodeFactory, new StoreCurrentDirectoryNodeFactory, new StoreSystemVariableNodeFactory));