<?php

	class NlToBrNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Convert newlines to breaks in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = nl2br($%variable);");
		}
	}
	
	class EscapeSpecialCharsNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Escape special chars in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = htmlspecialchars($%variable);");
		}
	}
	
	class StoreGetRequestNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Store GET parameter <b>%index</b> in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = isset(\$_GET[%index]) ? \$_GET[%index] : null;");
		}
	}
	
	class StorePostRequestNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Store POST parameter <b>%index</b> in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = isset(\$_POST[%index]) ? \$_POST[%index] : null;");
		}
	}
	
	class StoreServerVariableNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Store SERVER variable <b>%index</b> in <b>%variable</b>");
		}
		
		public function process()
		{
			return $this->format("$%variable = isset(\$_SERVER[%index]) ? \$_SERVER[%index] : null;");
		}
	}
	
	class HandleFormNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Handle form <b>%form</b>");
		}
		
		public function process()
		{
			return $this->format("if(isset(\$_GET[%form]))");
		}
	}
	
	class DownloadFileNode extends BaseNode
	{
		public function getDescription()
		{
			return $this->format("Downlaod file from <b>%url</b> and save to <b>%path</b>");
		}
		
		public function process()
		{
			return $this->format("file_put_contents(%path, file_get_contents(%url));");
		}
	}
	
	class StoreGetRequestNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Store GET parameter";
		}
		
		public function getGroup()
		{
			return "Web";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StoreGetRequestNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "index", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class StorePostRequestNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Store POST parameter";
		}
		
		public function getGroup()
		{
			return "Web";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StorePostRequestNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "index", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class StoreServerVariableNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Store server variable";
		}
		
		public function getGroup()
		{
			return "Web";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StoreServerVariableNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "index", "required" => true),
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class HandleFormNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Handle form";
		}
		
		public function getGroup()
		{
			return "Web";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new HandleFormNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "form", "required" => true),
			);
		}
	}
	
	class NlToBrNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Newlines to breaks";
		}
		
		public function getGroup()
		{
			return "Web";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new NlToBrNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class EscapeSpecialCharsNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Escape special characters";
		}
		
		public function getGroup()
		{
			return "Web";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new EscapeSpecialCharsNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "variable", "required" => true)
			);
		}
	}
	
	class DownloadFileNodeFactory extends NodeFactory
	{
		public function getName()
		{
			return "Download file";
		}
		
		public function getGroup()
		{
			return "Web";
		}
		
		public function instantiate($blueprint)
		{
			if($blueprint)
				return new StoreGetRequestNode($this, $blueprint);
			
			return null;
		}
		
		public function queryAttributeList()
		{
			return array(
				array("name" => "url", "required" => true),
				array("name" => "path", "required" => true)
			);
		}
	}
	
	NodeList::registerFactory(array(new StoreGetRequestNodeFactory, new StorePostRequestNodeFactory, new StoreServerVariableNodeFactory, new HandleFormNodeFactory, new DownloadFileNodeFactory));
	NodeList::registerFactory(array(new NlToBrNodeFactory, new EscapeSpecialCharsNodeFactory));